
/* LOG_FACILITY is passed as the third argument to the openlog() system
 * call.  You can change LOG_FACILITY to redirect the syslog entries
 * created by LSV_AMIN as you like.  If your system's openlog() doesn't
 * have a "facility" option, just set LOG_FACILITY to 0.
 */
#define LOG_FACILITY LOG_MAIL

/* MAX_BODY_BUFFER is the size (in K-bytes) of the internal buffer used
 * by LSV_AMIN when scanning mail messages for delivery error indicators.
 * If the buffer is exhausted while scanning, LSV_AMIN will create a
 * syslog entry to that effect.
 */
#ifndef MAX_BODY_BUFFER
#define MAX_BODY_BUFFER 80
#endif

/* MAX_BODY_LINES defines the number of lines LSV_AMIN will scan in a
 * mail delivery error looking for the cause and target userid.  If
 * you are receiving delivery errors with lots of chit-chat prepended
 * to the actual sendmail "transaction log", thenyou might have to
 * increase MAX_BODY_LINES (and possibly MAX_BODY_BUFFER).
 */
#ifndef MAX_BODY_LINES
#define MAX_BODY_LINES 60
#endif

/* If no spool directory is specified on the command line (either as
 * the first non-flagged argument, or following a "-s" flag), then
 * SPOOL_DIR will be used as the full pathname of the LISTSERV spool 
 * directory.  The Makefile delivered with LISTSERV will automatically
 * define this macro using the $LSVSPOOL symbol.  You can leave the
 * directory off your alias file entries now using flags to set the
 * other arguments.  For example,
 *
 * owner-test-l: "|/usr/listserv/lsv_amin -t owner-test-l"
 */
#ifndef SPOOL_DIR
#define SPOOL_DIR "/var/spool/listserv"
#endif

/* LISTSERV will prepend the ErrInsert string to the body of any
 * mail delivery errors it recognizes.
 */
#define ErrInsert \
"\n\nAn error was detected while processing the enclosed message. A list of\
\nthe affected recipients follows. This list is in a special format that\
\nallows software like LISTSERV to automatically take action on incorrect\
\naddresses.\n\n"

struct SM_Err_type {
  char *err_msg;
  int err_type;
};

/* SM_Err_type holds a list of "reasons" LSV_AMIN recognizes, and an
 * associated LISTSERV error code. LISTSERV currently recognizes the
 * following codes:
 *
 * 0 - Unknown or unclassifiable errors.
 * 1 - There is no such host, or the address is otherwise invalid.
 * 2 - Configuration error found trying to deliver the mail.
 * 3 - Non-existant user.
 *
 * Provided a mailing list is setup with Auto-Delete enabled, any
 * code 1 or code 3 errors result in the indicated address being
 * removed from the mailing list.  Therefore, only permanent and
 * unambiguous errors should be classified as type 1 or 3.  All
 * code 0 or 2 errors are simply forwarded to the list owner.  You
 * can add as many new entries as you like (in any order) so long
 * as the first error message in the list has a code of 0.
 */

struct SM_Err_type err_list[] =
{ { "*default*", 0},            /* this entry must be code 0... */
  { "Can't create output", 0},
  { "No space left on device", 0},
  { "Deffered: Connect refused", 0}, 
  { "Host unknown", 1},
  { "550 Host unknown", 1},
  { "error -- Bad system name", 1},
  { "Mail loop detected", 2},
  { "User unknown", 3},
  { "user unknown", 3}
};
